import subprocess

var("x,y")



def zero_vel_c(a, L, l=1.6):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	P = point([(0,0)],size=0)
	for k in L:
		c = f(x=lag[k][0], y=lag[k][1]) 
		P += implicit_plot(f-c, (x,-l,l), (y,-l,l), plot_points=400)
	return P



def cJa(Lor):
	a = Lor[0]
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	(f(x=Lor[1], y=Lor[2])-Lor[3]^2-Lor[4]^2 ).n()


def c1mu(a):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	f(x=lag[0][0], y=0) 

def c2mu(a):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	f(x=lag[1][0], y=0) 

def l_lag(mu):
	lag = [0,0,0, ((1-2*mu)/2, sqrt(3)/2), ((1-2*mu)/2, -sqrt(3)/2) ]
	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[0] = (find_root(fu, -mu,1-mu) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3-1)
	lag[1] = (find_root(fu, 1-mu, 2) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3+1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[2] = (find_root(fu, -2, -mu) , 0)
	
	return lag

def lag_p_m(mu):
	P = points([(0,0),(1,0)],size=0)
	P += point(l_lag(mu), size=20, color='red')
	P += point([(-mu,0), (1-mu,0)], size=50, color='green')
	return P

def single_zv(a, c, l=1.6):
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return lag_p_m(a)+implicit_plot(f-c, (x,-l,l), (y,-l,l), plot_points=400)

	
	
	
def run_and_plot(Lp=[-1.6,1.6,-1.6,1.6, './r3b.eps']):
	subprocess.call('./run_r3b', shell=True)

	print 'FILE CREATED'

	Ldat = []
	with open('./r3b.dat', 'r') as f:
		res_x = float((Lp[1]-Lp[0])/800.0)
		res_y = float((Lp[3]-Lp[2])/800.0)
		lxy = (10, 10)
		line=f.readline()
		mmu = float(line)
		line=f.readline()
		cJ = float(line)
		line=f.readline()
		nsteps = int(line)
		for k in range(nsteps):
			line=f.readline()
			xy = line.split(',')
			xy = (float(xy[0]),float(xy[1]))
			if abs(lxy[0]-xy[0])/res_x + abs(lxy[1]-xy[1])/res_y > 1:
				lxy = xy
				Ldat.append( xy )
	print 'DATA LOADED IN LIST'
	P = lag_p_m(mmu)
	f = x^2 + y^2 +2*((1-mmu)/sqrt((x+mmu)^2+y^2)+mmu/sqrt((x-1+mmu)^2+y^2))
	P += implicit_plot(f-cJ, (x,Lp[0],Lp[1]), (y,Lp[2],Lp[3]), plot_points=400, linestyle='--', color='red')
	P += list_plot(Ldat, plotjoined=True, thickness=1)
	#P += list_plot(Ldat)
	P.set_aspect_ratio(1)
	#P.axes(False)
	P.fontsize(15)
	P.save(Lp[4], xmin=Lp[0], xmax=Lp[1], ymin=Lp[2], ymax=Lp[3], figsize=5)



def data_to_file(Lor, Lnu):
	names = ['MU','X0','Y0','VX0','VY0', 'N_ORB','DT']
	if (len(Lor)!=5) or (len(Lnu)!=2): 
		print 'Error in the number of parameters of the orbit'
		return
	with open('./r3b.h', 'w') as f:
		f.write('// AUTO-GENERATED FILE\n\n')
		f.write('/* Parameters of the orbit */\n')
		for k in range(5):
			f.write('#define '+names[k]+' '+str(Lor[k])+'\n')
		f.write('\n/* Parameters of the numerical method */\n')
		for k in range(2):
			f.write('#define '+names[k+5]+' '+str(Lnu[k])+'\n')

def orbit( Lor, Lnu, Lpl=[-1.6,1.6,-1.6,1.6,'./r3b.eps']):
	pcs(Lor, Lpl)
	data_to_file(Lor, Lnu)
	run_and_plot(Lpl)
	
	
def pcs(Lor,Lp):	
	print '\n'+Lp[-1]
	print 'Lorb =', [k.n(digits=6) for k in Lor]
	print 'c1 =', c1mu(Lor[0])
	print 'cJ =', cJa(Lor)
	print 'c2 =', c2mu(Lor[0])
	print 'c1-cJ =',c1mu(Lor[0])-cJa(Lor)
	print 'c2-cJ =',c2mu(Lor[0])-cJa(Lor)
	print  '============='


	
def plot1():
	Lmu = [0.4, 0.3, 0.1, 0.01]
	for k in range(len(Lmu)):
		mu = Lmu[k]
		P = lag_p_m(mu)
		P += zero_vel_c(mu, [0], 1.2)
		P.set_aspect_ratio(1)
		P.axes(False)
		P.fontsize(15)
		P.save('../../images/cl1'+str(k)+'.png', figsize = 5 )

def plot2():
	h = 0.5/80
	L = [(xx, (3+(9*xx)^(2/3)-10*xx/3-c1mu(xx)).n()/c1mu(xx)) for xx in srange(0.0,0.5,h)]
	P = list_plot(L, plotjoined=True, thickness=2)
	P.fontsize(15)
	P.save('../../images/rel_err.png', xmax=0.51, figsize = 5 )

def tabu1():
	mu = 0.3
	d = 6
	f = 3+(9*x)^(2/3)-10*x/3
	print c1mu(mu).n(digits = d), f(x=mu).n(digits = d)

	Lmu = [0.5, 0.4, 0.3, 0.2, 0.1, 0.01]
	for mu in  Lmu:
		s = '$c_1('+str(mu.n(digits=2))+ r')$&$=$ $'+str(c1mu(mu).n(digits = d))+'$'
		print s
		print '\\\\'
		s = r'&$\sim$ $'+str(f(x=mu).n(digits = d))+'$'
		print s
		print '\\\\[3.4161pt]'
	f = 2*x*(1-x^2)^2 -(2+2*x)*(1+x^2)*(1-x)^2 -8*x*(1-x)^2 +(2+2*x)*(1+x^2)*(1+x)^2 -8*x*(1+x)^2

def plot3():
	mu = 0.2
	#print mu
	#print 'c1mu =',c1mu(mu) 
	#print 'c2mu =',c2mu(mu) 

	LcJ = [3.9, 3.7, 3.5]
	for k in range(3):
		P = single_zv(mu, LcJ[k], l=1.7)
		P.set_aspect_ratio(1)
		#P.axes(False)
		P.fontsize(15)
		P.save('../../images/conf'+str(k)+'.png', figsize = 5 )


def plot4():
	# SUCESIVE CAPTURES asymmetric
	Lorb = [0.4,	0.0, 0.0,	0.6,0.12]
	Lnum = [4.0, 0.001]
	Lplo = [-1.0,1.0,-0.5,0.5,'../../images/ocapas.png']
	orbit( Lorb, Lnum, Lplo )
	# CAPTURE ESCAPE
	Lorb = [0.29,	0.1, 0.0,	0.85,0.57]
	Lnum = [4.9, 0.0001]
	Lplo = [-0.9,1.7,-0.7,0.6,'../../images/ocapesc.png']
	orbit( Lorb, Lnum, Lplo )
	# SUCESIVE CAPTURES symmetric
	Lorb = [0.5,	0.0, 0.0,	0.5,0.0]
	Lnum = [5.0, 0.0001]
	Lplo = [-1.0,1.0,-0.5,0.5,'../../images/ocaps.png']
	orbit( Lorb, Lnum, Lplo )
	# 1-CONFINED ring
	Lorb = [0.2,	0.1, 0.08,	-0.532668, 1.16390]
	Lnum = [3.0, 0.0001]
	Lplo = [-0.68,1.05,-0.5,0.5,'../../images/o1confr.png']
	orbit( Lorb, Lnum, Lplo )
	# 1-CONFINED convex
	Lorb = [0.2,	0.1, 0.08,	0.691587, 1.07708]
	Lnum = [3.0, 0.0001]
	Lplo = [-0.68,1.05,-0.5,0.5,'../../images/o1confcx.png']
	orbit( Lorb, Lnum, Lplo )
	# 1-CONFINED concave
	Lorb = [0.2,	0.1, 0.08,	1.12331, 0.613652]
	Lnum = [3.0, 0.0001]
	Lplo = [-0.68,1.05,-0.5,0.5,'../../images/o1confcv.png']
	orbit( Lorb, Lnum, Lplo )
	# 1-CONFINED second mass
	Lorb = [0.2,	0.9, 0.0,	1.29144, 0.705515]
	Lnum = [3.0, 0.0001]
	Lplo = [-0.68,1.05,-0.5,0.5,'../../images/o1conf2m.png']
	orbit( Lorb, Lnum, Lplo )

		
#tabu1()

plot1()
plot2()
plot3()
plot4()










# PURGE
Lorb = [0.25,	40.2, -0.25,	0.1,0.57]
Lnum = [0.2, 0.1]
orbit( Lorb, Lnum )

subprocess.call('./purge_r3b', shell=True)
