import subprocess
import os

def l_lag(mu):
	lag = [0,0,0, ((1-2*mu)/2, sqrt(3)/2), ((1-2*mu)/2, -sqrt(3)/2) ]
	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[0] = (find_root(fu, -mu,1-mu) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3-1)
	lag[1] = (find_root(fu, 1-mu, 2) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3+1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[2] = (find_root(fu, -2, -mu) , 0)
	
	return lag

def rota(L,th):
	Lc = L[:] 
	for k in range(len(L)):
		Lc[k] = (L[k][0]*cos(th)-L[k][1]*sin(th), L[k][0]*sin(th)+L[k][1]*cos(th) ) 
	return Lc

def data_to_file(Lor, Lnu):
	names = ['MU','X0','Y0','VX0','VY0', 'N_ORB','DT']
	if (len(Lor)!=5) or (len(Lnu)!=2): 
		print 'Error in the number of parameters of the orbit'
		return
	with open('./r3b.h', 'w') as f:
		f.write('// AUTO-GENERATED FILE\n\n')
		f.write('/* Parameters of the orbit */\n')
		for k in range(5):
			f.write('#define '+names[k]+' '+str(Lor[k])+'\n')
		f.write('\n/* Parameters of the numerical method */\n')
		for k in range(2):
			f.write('#define '+names[k+5]+' '+str(Lnu[k])+'\n')

	
def retrieve_data_frames(Lorb, Lnum, nfr):
	data_to_file(Lorb, Lnum)
	subprocess.call('./run_r3b', shell=True)

	print 'FILE CREATED'

	Ldat = []
	with open('./r3b.dat', 'r') as f:
		line=f.readline()
		mmu = float(line)
		line=f.readline()
		cJ = float(line)
		line=f.readline()
		nsteps = int(line)
		#print nsteps
		k = 0
		#retrieve only nfr values
		for nf in range(nfr):
			ind = floor(nf*nsteps/nfr)
			while(k!=ind):
				line = f.readline()
				k +=1
			line = f.readline()
			xy = line.split(',')
			a, b = float(xy[0]),float(xy[1])
			t = k*Lnum[1]
			Ldat.append( (t,(a*cos(t)-b*sin(t),a*sin(t)+b*cos(t))) )
			k += 1
	print 'DATA LOADED IN LIST'
	return Ldat

def make_frames( mmu, Ld, Lp, f_lag):
	nfr = len(Ld)
	lag = l_lag(mmu)
	mas = [(-mmu,0),(1-mmu,0)] 

	# SUPERPOSITION OF FRAMES
	P = point([(0,0)],size=0)
	for k in srange(nfr):
		P += point(rota(mas,Ld[k][0]), color='green')
	P += list_plot([item[1] for item in Ld])
	P.set_aspect_ratio(1)
	if f_lag&2==0: P.axes(False)
	else: P.axes(True)
	P.fontsize(15)
	P.save(Lp[4], xmin=Lp[0], xmax=Lp[1], ymin=Lp[2], ymax=Lp[3])

	# INDIVIDUAL FRAMES
	for k in srange(nfr):
		print k+1, 'out of', nfr
		P = point([(0,0)],size=0)
		P += point(rota(mas,Ld[k][0]), color='green', size=60)
		if f_lag&1==1:
			P += point(rota(lag,Ld[k][0]), color='red', size=10)
		P += point(Ld[k][1], size=60)
		P.set_aspect_ratio(1)
		if f_lag&2==0: P.axes(False)
		else: P.axes(True)
		P.fontsize(15)
		P.save('temp_frame'+str(k).zfill(3)+'.png', xmin=Lp[0], xmax=Lp[1], ymin=Lp[2], ymax=Lp[3])

	
def make_animation(Lorb, Lnum, Lplo, nfr, dela=100, f_lag = False):
	Ldat = retrieve_data_frames(Lorb, Lnum, nfr)
	make_frames( Lorb[0], Ldat, Lplo, f_lag)
	filen = os.path.splitext(Lplo[-1])[0]
	subprocess.call('./animate '+str(dela)+' '+filen, shell=True)

	
# CAPTURE ESCAPE
Lorb = [0.29,	 0.5427008, -0.2574309,  -0.1441290, -0.5512641]
Lnum = [0.2955, 0.0001]
Lplo = [-0.57,0.7,-0.55,0.7,'../../images/frames.eps']
make_animation(Lorb, Lnum, Lplo, 100, 20, 3)


# KEPLER
Lorb = [0.1,	0.0, 1.2,	0.1,0.0]
Lnum = [0.5, 0.01]
Lplo = [-3,3,-3.5,1.5,'./r3b.eps']
#make_animation(Lorb, Lnum, Lplo, 10, 100, 3)


# PERIODIC trian
Lorb = [0.002,	-0.9, 0.0,	0.0,-0.058]
Lnum = [2.0, 0.01]
Lplo = [-1.2,1.2,-1.2,1.2,'./r3b.eps']
#make_animation(Lorb, Lnum, Lplo, 300, 25, 3)


# PURGE
Lorb = [0.29,	 0.5427008, -0.2574309,  -0.1441290, -0.5512641]
Lnum = [0.2955, 0.1]
Lplo = [-0.57,0.7,-0.55,0.7,'./r3b.eps']
#make_animation(Lorb, Lnum, Lplo, 4, 20, 3)


